{% macro trigger_menu(is_freerun) -%}
<div class="context_menu_hide" id="rmenu">
    <ul>
        <li>
            <a onclick="process_batch('action/activate_trigger', {{ is_freerun|string|lower }} )">
                <i class="fa fa-toggle-on"></i>&nbsp;Activate
            </a>
        </li>
        <li>
            <a onclick="process_batch('action/deactivate_trigger', {{ is_freerun|string|lower }} )">
                <i class="fa fa-toggle-off"></i>&nbsp;Deactivate
            </a>
        </li>
    </ul>
</div>
<script type="text/javascript" src="{{ url_for('static', file='js/context_menu.js') }}"></script>
{%- endmacro %}


{% macro job_menu() -%}
<div class="context_menu_hide" id="rmenu">
    <ul>
        <li>
            <a onclick="process_batch('action/skip', false)">
                <i class="fa fa-step-forward"></i>&nbsp;Skip
            </a>
        </li>
        <li>
            <a onclick="process_batch('action/reprocess', false)">
                <i class="fa fa-repeat"></i>&nbsp;Reprocess
            </a>
        </li>
    </ul>
</div>
<script type="text/javascript" src="{{ url_for('static', file='js/context_menu.js') }}"></script>
{%- endmacro %}
